import os
import pathlib
import shutil
import urllib.parse
import zipfile
from os.path import exists
from pathlib import Path
from urllib.parse import urlparse

import requests

import helpers
import sfc_core_requests
import variables
from config import Config


def check_update_url():
    update_url = Config.UPDATE_URL
    system_version = variables.SYSTEM_VERSION
    next_update_version = sfc_core_requests.SfcRequests().check_updates()

    if update_url and system_version and not system_version == next_update_version:
        file_name = '{next_update_version}.zip'.format(next_update_version=next_update_version)
        final_url = urllib.parse.urljoin(update_url, file_name)
        if check_url(final_url):
            return final_url, next_update_version

    return None, None


def create_startup_update_file(update_path):
    update_path = update_path.joinpath('sfc-machine-integration')
    file_path = update_path.joinpath('startup_update.bat').resolve()
    virtual_env = os.environ.get('VIRTUAL_ENV')
    if virtual_env:
        virtual_env = pathlib.Path(virtual_env).joinpath('Scripts').resolve()

    cmd_str = '"{virtual_env} cd /d {update_path} & python main.py & exit"'.format(
        virtual_env='cd /d {} & activate &'.format(virtual_env),
        update_path=update_path
    )
    file_content = '@echo off\ncmd /k {cmd_str}\n'.format(cmd_str=cmd_str)

    with open(file_path, 'w+') as file:
        file.write(file_content)
        return file_path


def get_update():
    url, next_version = check_update_url()
    root_dir = os.environ.get('ROOT_DIR')

    if url and next_version and root_dir:
        temp_path = pathlib.Path(root_dir).joinpath(next_version).resolve()
        exe_file = os.environ.get('EXECUTABLE_FILE')

        if exe_file:
            Path(temp_path).mkdir(parents=True, exist_ok=True)
            file_name = os.path.basename(urlparse(url).path)
            temp_file_path = temp_path.joinpath(file_name).resolve()

            try:
                response = requests.get(url, timeout=3, verify=False)
                response.raise_for_status()
                os.environ['UPDATE_MODE'] = 'True'

                with open(temp_file_path, 'wb') as f:
                    f.write(response.content)

                    with zipfile.ZipFile(temp_file_path, 'r') as zip_ref:
                        zip_ref.extractall(temp_path)
                        file_update_path = create_startup_update_file(temp_path)
                        helpers.recursive_chmod([root_dir, ])
                        os.startfile(os.path.normpath(file_update_path))

            except requests.exceptions.HTTPError as e:
                os.environ['UPDATE_MODE'] = 'False'
                print('Check updates: ', e)

        else:
            os.environ['UPDATE_MODE'] = 'False'
    else:
        os.environ['UPDATE_MODE'] = 'False'


def change_version():
    root_dir = os.environ.get('ROOT_DIR')
    new_version_dir = pathlib.Path(__file__).parent.resolve()

    if root_dir:
        root_dir = pathlib.Path(root_dir).resolve()
        main_dir = root_dir.joinpath('sfc-machine-integration').resolve()

        helpers.recursive_chmod([main_dir, ])

        shutil.rmtree(main_dir, ignore_errors=True)
        shutil.copytree(new_version_dir, main_dir, dirs_exist_ok=True)

        file_update_path = main_dir.joinpath('startup_update.bat')
        if exists(file_update_path):
            os.remove(file_update_path)

        os.environ['TEMP_UPDATE_PATH'] = str(new_version_dir)
        os.environ['UPDATE_MODE'] = 'False'
        os.environ['UPDATE_MODE_DONE'] = 'True'
        helpers.recursive_chmod([root_dir, ])  # da permissao a todo diretorio

        exe_file = os.environ.get('EXECUTABLE_FILE')
        file_startup_path = main_dir.joinpath(exe_file).resolve()
        os.startfile(os.path.normpath(file_startup_path))


def check_url(url):
    try:
        response = requests.head(url, timeout=3)
        return response.status_code == 200
    except Exception:
        return False
